@@ -1,10 +1,10 @@
 package Net::DNS::Nameserver;
 
 #
-# $Id: Nameserver.pm 1019 2012-10-04 08:29:09Z willem $
+# $Id: Nameserver.pm 1118 2013-09-24 20:27:18Z willem $
 #
 use vars qw($VERSION);
-$VERSION = (qw$LastChangedRevision: 1019 $)[1];
+$VERSION = (qw$LastChangedRevision: 1118 $)[1]; # Unchanged since 1046
 
 
 =head1 NAME
@@ -195,11 +195,10 @@ sub make_reply {
 	}
 
 	if ( $query->header->qr() ) {
-		print "ERROR: invalid packet (qr was set, dropping)\n" if $self->{Verbose};
+		print "ERROR: invalid packet (qr set), dropping\n" if $self->{Verbose};
 		return;
 	}
 
-
 	my $reply  = $query->reply();
 	my $header = $reply->header;
 	my $headermask;
@@ -351,11 +350,16 @@ sub tcp_connection {
 			my $qbuf = substr( $self->{_tcp}{$sock}{inbuffer}, 0, $self->{_tcp}{$sock}{querylength} );
 			substr( $self->{_tcp}{$sock}{inbuffer}, 0, $self->{_tcp}{$sock}{querylength} ) = "";
 			my $query = new Net::DNS::Packet( \$qbuf );
-			my $conn  = {
-				sockhost => $sock->sockhost(),
-				sockport => $sock->sockport(),
-				peerhost => $sock->peerhost(),
-				peerport => $sock->peerport()};
+			if ( my $err = $@ ) {
+				print "Error decoding query packet: $err\n" if $self->{Verbose};
+				undef $query;			# force FORMERR reply
+			}
+			my $conn = {
+				sockhost => $sock->sockhost,
+				sockport => $sock->sockport,
+				peerhost => $sock->peerhost,
+				peerport => $sock->peerport
+				};
 			my $reply = $self->make_reply( $query, $sock->peerhost, $conn );
 			if ( not defined $reply ) {
 				print "I couldn't create a reply for $peer. Closing socket.\n"
@@ -398,10 +402,9 @@ sub udp_connection {
 	print "UDP connection from $peerhost:$peerport to $sockhost\n" if $self->{Verbose};
 
 	my $query = new Net::DNS::Packet( \$buf, $self->{Verbose} );
-	my $err = $@;
-	if ( !defined($query) ) {
-		print "Error interpreting query: $err\n" if $self->{Verbose};
-		return;
+	if ( my $err = $@ ) {
+		print "Error decoding query packet: $err\n" if $self->{Verbose};
+		undef $query;					# force FORMERR reply
 	}
 	my $conn = {
 		sockhost => $sock->sockhost,
@@ -416,7 +419,7 @@ sub udp_connection {
 	local $| = 1 if $self->{Verbose};
 	print "Writing response - " if $self->{Verbose};
 
-	if ( $sock->send( $reply->data ) ) {			#
+	if ( $sock->send( $reply->data ) ) {
 		print "done\n" if $self->{Verbose};
 	} else {
 		print "failed to send reply: $!\n" if $self->{Verbose};
@@ -516,7 +519,7 @@ sub loop_once {
 					# victims of DoS (client sending lots of queries and never reading
 					# from it's socket).
 					# Note that this does not disable serialisation on part of the
-					# client. The split second it should take for us to lookip the
+					# client. The split second it should take for us to lookup the
 					# next query, is likely faster than the time it takes to
 					# send the response... well, unless it's a lot of tiny queries,
 					# in which case we will be generating an entire TCP packet per
@@ -593,7 +596,7 @@ Creates a nameserver object.  Attributes are:
     Verbose		Print info about received
 			queries.			Defaults to 0 (off).
     Truncate		Truncates UDP packets that
-			are to big for the reply	Defaults to 1 (on)
+			are too big for the reply	Defaults to 1 (on)
     IdleTimeout		TCP clients are disconnected
 			if they are idle longer than
 			this duration.			Defaults to 120 (secs)
